package org.bdware.sc;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.bdware.sc.bean.Contract;

import java.io.*;
import java.nio.file.Files;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class YPKResourceManager {
    public static String ypkCacheDir = "./ypkcache";
    static ExecutorService es = Executors.newFixedThreadPool(2);

    public static void unzipYPK(Contract contract) {
        es.execute(() -> unzip(contract.getScriptStr(), getUnzipDir(contract)));
    }

    public static String getUnzipDir(Contract contract) {
        return ypkCacheDir + "/" + contract.getID();
    }

    private static final long MAX_CACHE_SIZE = 2L * 1024 * 1024 * 1024; // 2GB
    private static final long MAX_FILE_SIZE = 10 * 1024 * 1024; // 10MB

    private static final Cache<String, byte[]> cache = Caffeine.newBuilder()
            .maximumSize(MAX_CACHE_SIZE).expireAfterAccess(30, TimeUnit.MINUTES).build();

    public static InputStream getCachedStream(Contract c, String path) throws IOException {
        File target = new File(getUnzipDir(c), path);
        String key = target.getAbsolutePath();

        if (target.length() > MAX_FILE_SIZE) {
            // 文件过大，直接从文件系统读取
            return new FileInputStream(target);
        } else {
            byte[] data = cache.get(key, k -> {
                try {
                    return Files.readAllBytes(target.toPath());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            });
            // 从缓存中返回一个新的 ByteArrayInputStream
            return new ByteArrayInputStream(data);
        }
    }

    public static void unzip(String zipFilePath, String destDir) {
        File dir = new File(destDir);
        if (!dir.exists())
            dir.mkdirs();
        byte[] buffer = new byte[1024];
        try {
            ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFilePath));
            ZipEntry zipEntry = zis.getNextEntry();
            while (zipEntry != null) {
                String fileName = zipEntry.getName();
                File newFile = new File(destDir + File.separator + fileName);
                // 创建所有上级目录
                new File(newFile.getParent()).mkdirs();

                if (zipEntry.isDirectory()) {
                    if (!newFile.isDirectory() && !newFile.mkdirs()) {
                        throw new IOException("Failed to create directory " + newFile);
                    }
                } else {
                    // 写入文件内容
                    FileOutputStream fos = new FileOutputStream(newFile);
                    int len;
                    while ((len = zis.read(buffer)) > 0) {
                        fos.write(buffer, 0, len);
                    }
                    fos.close();
                }
                zipEntry = zis.getNextEntry();
            }
            zis.closeEntry();
            zis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static boolean existYPK(Contract contract) {
        return new File(contract.getScriptStr()).exists();
    }
}
